<?php

declare(strict_types=1);

namespace Drupal\oktaplus;

use Drupal\oktaplus\Enum\EventOutcome;
use Drupal\oktaplus\Enum\EventType;
use Drupal\oktaplus\Exception\InvalidLogEventException;
use Drupal\oktaplus\Exception\UnhandledLogEventTypeException;

/**
 * Represents an Okta event.
 *
 * @see https://developer.okta.com/docs/reference/api/system-log/#logevent-object
 */
final readonly class OktaLogEvent {

  public function __construct(
    public string $uuid,
    public \DateTimeInterface $published,
    public EventType $eventType,
    public EventOutcome $eventOutcome,
    public array $payload,
  ) {}

  /**
   * Create an Okta log event from an array.
   *
   * @throws \Drupal\oktaplus\Exception\InvalidLogEventException
   * @throws \Drupal\oktaplus\Exception\UnhandledLogEventTypeException
   */
  public static function fromArray(array $data): self {
    $uuid = $data['uuid'] ?? throw new InvalidLogEventException('Missing event UUID.');
    if (!isset($data['eventType'])) {
      throw new InvalidLogEventException('Missing eventType. Event UUID: ' . $uuid);
    }
    $eventType = EventType::tryFrom($data['eventType']);
    if ($eventType === NULL) {
      throw new UnhandledLogEventTypeException(\sprintf('Unhandled event type %s. Event UUID: %s', $data['eventType'], $uuid));
    }
    if (!isset($data['published'])) {
      throw new InvalidLogEventException('Missing published date. Event UUID: ' . $uuid);
    }
    $published = \DateTimeImmutable::createFromFormat('Y-m-d\TH:i:s.u\Z', $data['published']);
    if ($published === FALSE) {
      throw new InvalidLogEventException(\sprintf('Invalid date format %s. Event UUID: %s', $data['published'], $uuid));
    }
    $outcome = EventOutcome::tryFrom($data['outcome']['result'] ?? '');
    if ($outcome === NULL) {
      throw new InvalidLogEventException(\sprintf('Missing outcome. Event UUID: %s', $uuid));
    }
    return new self($uuid, $published, $eventType, $outcome, $data);
  }

  /**
   * Gets the target app.
   */
  public function getTargetAppInstance(): ?array {
    return $this->getTarget('AppInstance');
  }

  /**
   * Gets the target user.
   */
  public function getTargetUser(): ?array {
    return $this->getTarget('User');
  }

  /**
   * Gets the target group.
   */
  public function getTargetGroup(): ?array {
    return $this->getTarget('UserGroup');
  }

  /**
   * Gets any target.
   */
  private function getTarget(string $type): ?array {
    if (!isset($this->payload['target']) || !\is_array($this->payload['target'])) {
      return [];
    }
    return \array_find(
      $this->payload['target'],
      static fn (array $target): bool => ($target['type'] ?? '') === $type,
    );
  }

}
