<?php

declare(strict_types=1);

namespace Drupal\oktaplus;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ConfigValueException;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\externalauth\AuthmapInterface;
use Drupal\oktaplus\Event\UserActivatedEvent;
use Drupal\oktaplus\Event\UserBlockedEvent;
use Drupal\oktaplus\Event\UserRolesAddedEvent;
use Drupal\oktaplus\Event\UserRolesRemovedEvent;
use Drupal\oktaplus\Form\OktaplusSettingsForm;
use Drupal\openid_connect\OpenIDConnect;
use Drupal\user\UserInterface;
use Symfony\Component\DependencyInjection\Attribute\Autowire;
use Symfony\Contracts\EventDispatcher\EventDispatcherInterface;

class OktaplusUserUtility {

  public function __construct(
    protected readonly ConfigFactoryInterface $configFactory,
    protected readonly EntityTypeManagerInterface $entityTypeManager,
    #[Autowire(service: 'externalauth.authmap')]
    protected readonly AuthmapInterface $authmap,
    protected readonly EventDispatcherInterface $eventDispatcher,
    #[Autowire(service: 'openid_connect.openid_connect')]
    protected readonly ?OpenIDConnect $oidc = NULL,
  ) {}

  /**
   * Get the Drupal user account from target user in the event.
   */
  public function getTargetUser(OktaLogEvent $event): ?UserInterface {
    $targetUser = $event->getTargetUser();
    if (!isset($targetUser['id'])) {
      return NULL;
    }
    $uid = $this->authmap->getUid($targetUser['id'], $this->getAuthmapProvider());
    if ($uid === FALSE) {
      return NULL;
    }
    $storage = $this->entityTypeManager->getStorage('user');
    /** @var \Drupal\user\UserInterface */
    return $storage->load($uid);
  }

  /**
   * Creates a user from an event.
   */
  public function createUserFromEvent(OktaLogEvent $event): ?UserInterface {
    if ($this->oidc === NULL) {
      throw new \RuntimeException('OIDC service not available.');
    }
    $target = $event->getTargetUser();
    if ($target === NULL) {
      return NULL;
    }
    return $this->oidc->createUser(
      sub: $target['id'],
      userinfo: [
        'name' => $target['displayName'],
        'email' => $target['alternateId'],
        'preferred_username' => $target['alternateId'],
      ],
      client_name: $this->getOktaplusClient() ?? throw new \RuntimeException('Oktaplus client not configured.'),
      status: 0,
    );
  }

  /**
   * Activates a user.
   */
  public function activateUser(UserInterface $user, ?OktaLogEvent $event): void {
    if ($event !== NULL) {
      $this->eventDispatcher->dispatch(new UserActivatedEvent($user, $event));
    }
    $user->activate()->save();
  }

  /**
   * Blocks a user.
   */
  public function blockUser(UserInterface $user, ?OktaLogEvent $event): void {
    if ($event !== NULL) {
      $this->eventDispatcher->dispatch(new UserBlockedEvent($user, $event));
    }
    $user->block()->save();
  }

  /**
   * Deletes a user from the authmap.
   */
  public function deleteUserFromAuthmap(UserInterface $user): void {
    $this->authmap->delete((int) $user->id(), $this->getAuthmapProvider());
  }

  /**
   * Adds one or more roles to a user.
   */
  public function addUserRoles(UserInterface $user, array $roles, ?OktaLogEvent $event): void {
    foreach ($roles as $role) {
      $user->addRole($role);
    }
    if ($event !== NULL) {
      $this->eventDispatcher->dispatch(new UserRolesAddedEvent($user, $roles, $event));
    }
    $user->save();
  }

  /**
   * Removes one or more roles from a user.
   */
  public function removeUserRoles(UserInterface $user, array $roles, ?OktaLogEvent $event): void {
    foreach ($roles as $role) {
      $user->removeRole($role);
    }
    if ($event !== NULL) {
      $this->eventDispatcher->dispatch(new UserRolesRemovedEvent($user, $roles, $event));
    }
    $user->save();
  }

  /**
   * Get the authmap provider.
   *
   * @throws \Drupal\Core\Config\ConfigValueException
   *   Thrown when no OpenIDConnect client is configured.
   */
  protected function getAuthmapProvider(): string {
    $client = $this->getOktaplusClient();
    if ($client === NULL) {
      throw new ConfigValueException('No OpenIDConnect client configured.');
    }
    return 'openid_connect.' . $client;
  }

  /**
   * Gets the configured client.
   */
  protected function getOktaplusClient(): ?string {
    $config = $this->configFactory->get(OktaPlusSettingsForm::CONFIG_NAME);
    return $config->get('client');
  }

}
