<?php

declare(strict_types=1);

namespace Drupal\Tests\oktaplus\Kernel;

use Drupal\Core\DependencyInjection\ContainerBuilder;
use Drupal\KernelTests\KernelTestBase;
use Drupal\oktaplus\Enum\EventOutcome;
use Drupal\oktaplus\Enum\EventType;
use Drupal\oktaplus\Event\UserActivatedEvent;
use Drupal\oktaplus\Event\UserBlockedEvent;
use Drupal\oktaplus\Event\UserRolesAddedEvent;
use Drupal\oktaplus\Event\UserRolesRemovedEvent;
use Drupal\oktaplus\OktaLogEvent;
use Drupal\oktaplus\OktaplusUserUtility;
use Drupal\user\UserInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Contracts\EventDispatcher\Event;

final class EventTest extends KernelTestBase implements EventSubscriberInterface {

  protected static $modules = ['system', 'oktaplus', 'externalauth'];

  /**
   * @var \Symfony\Contracts\EventDispatcher\Event[]
   */
  private array $caughtEvents = [];

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      UserActivatedEvent::class => 'catchEvent',
      UserBlockedEvent::class => 'catchEvent',
      UserRolesAddedEvent::class => 'catchEvent',
      UserRolesRemovedEvent::class => 'catchEvent',
    ];
  }

  /**
   * Push events to the caught events array.
   */
  public function catchEvent(Event $event): void {
    $this->caughtEvents[] = $event;
  }

  /**
   * {@inheritdoc}
   */
  public function register(ContainerBuilder $container): void {
    parent::register($container);
    $container->register(self::class)->setAutoconfigured(TRUE);
    $container->set(self::class, $this);
  }

  /**
   * @see \Drupal\oktaplus\Event\UserActivatedEvent
   */
  public function testUserActivatedEvent(): void {
    $oktaplus = $this->container->get(OktaplusUserUtility::class);
    $user = $this->createMock(UserInterface::class);
    $user->expects($this->once())
      ->method('activate')
      ->willReturnSelf();
    $oktaplus->activateUser($user, OktaLogEvent::fromArray([
      'uuid' => '4cc38455-c34f-4546-a436-9902c3f7b8ff',
      'eventType' => EventType::ApplicationUserMembershipAdd->value,
      'published' => '2021-01-01T00:00:00.000Z',
      'outcome' => [
        'result' => EventOutcome::Success->value,
      ],
    ]));
    $this->assertCount(1, $this->caughtEvents);
    $this->assertInstanceOf(UserActivatedEvent::class, $this->caughtEvents[0]);
  }

  /**
   * @see \Drupal\oktaplus\Event\UserBlockedEvent
   */
  public function testUserBlockedEvent(): void {
    $oktaplus = $this->container->get(OktaplusUserUtility::class);
    $user = $this->createMock(UserInterface::class);
    $user->expects($this->once())
      ->method('block')
      ->willReturnSelf();
    $oktaplus->blockUser($user, OktaLogEvent::fromArray([
      'uuid' => '4cc38455-c34f-4546-a436-9902c3f7b8ff',
      'eventType' => EventType::UserLifecycleDeactivate->value,
      'published' => '2021-01-01T00:00:00.000Z',
      'outcome' => [
        'result' => EventOutcome::Success->value,
      ],
    ]));
    $this->assertCount(1, $this->caughtEvents);
    $this->assertInstanceOf(UserBlockedEvent::class, $this->caughtEvents[0]);
  }

  /**
   * @see \Drupal\oktaplus\Event\UserRolesAddedEvent
   */
  public function testUserRolesAddedEvent(): void {
    $oktaplus = $this->container->get(OktaplusUserUtility::class);
    $user = $this->createMock(UserInterface::class);
    $user->expects($this->once())
      ->method('addRole')
      ->with('customer');
    $oktaplus->addUserRoles($user, ['customer'], OktaLogEvent::fromArray([
      'uuid' => '4cc38455-c34f-4546-a436-9902c3f7b8ff',
      'eventType' => EventType::GroupUserMembershipAdd->value,
      'published' => '2021-01-01T00:00:00.000Z',
      'outcome' => [
        'result' => EventOutcome::Success->value,
      ],
    ]));
    $this->assertCount(1, $this->caughtEvents);
    $this->assertInstanceOf(UserRolesAddedEvent::class, $this->caughtEvents[0]);
  }

  /**
   * @see \Drupal\oktaplus\Event\UserRolesRemovedEvent
   */
  public function testUserRolesRemovedEvent(): void {
    $oktaplus = $this->container->get(OktaplusUserUtility::class);
    $user = $this->createMock(UserInterface::class);
    $user->expects($this->once())
      ->method('removeRole')
      ->with('customer');
    $oktaplus->removeUserRoles($user, ['customer'], OktaLogEvent::fromArray([
      'uuid' => '4cc38455-c34f-4546-a436-9902c3f7b8ff',
      'eventType' => EventType::GroupUserMembershipRemove->value,
      'published' => '2021-01-01T00:00:00.000Z',
      'outcome' => [
        'result' => EventOutcome::Success->value,
      ],
    ]));
    $this->assertCount(1, $this->caughtEvents);
    $this->assertInstanceOf(UserRolesRemovedEvent::class, $this->caughtEvents[0]);
  }

}
