<?php

declare(strict_types=1);

namespace Drupal\Tests\oktaplus\Kernel;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Config\Config;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Queue\DatabaseQueue;
use Drupal\Core\Queue\QueueDatabaseFactory;
use Drupal\oktaplus\Controller\EventHookController;
use Drupal\oktaplus\Form\OktaplusSettingsForm;
use Drupal\oktaplus\Plugin\QueueWorker\OktaplusEventQueue;
use Drupal\openid_connect\OpenIDConnectClientEntityInterface;
use Drupal\Tests\UnitTestCase;
use PHPUnit\Framework\MockObject\MockObject;
use Symfony\Component\HttpFoundation\HeaderBag;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;

/**
 * @coversDefaultClass \Drupal\oktaplus\Controller\EventHookController
 */
final class EventHookControllerTest extends UnitTestCase {

  protected const CLIENT_ID = 'test_client';
  protected const EVENT_HOOK_SECRET = 'YWRtaW46cGFzc3dvcmQ=';

  /**
   * @covers ::handleVerification
   * @dataProvider providerHandleVerification
   * @phpstan-param class-string<\Throwable>|null $expectedException
   */
  public function testHandleVerification(string $clientId, array $headers, int $expectedStatusCode, string $expectedContent, ?string $expectedException = NULL): void {
    $client = $this->getClientMock($clientId);
    $request = new Request();
    $request->headers = new HeaderBag($headers);

    if ($expectedException !== NULL) {
      $this->expectException($expectedException);
    }

    $controller = $this->getEventHookController();
    $response = $controller->handleVerification($request);
    $this->assertSame($expectedStatusCode, $response->getStatusCode());
    $this->assertSame($expectedContent, $response->getContent());
  }

  /**
   * @covers ::handleEvent
   * @dataProvider providerHandleEvent
   * @phpstan-param class-string<\Throwable>|null $expectedException
   */
  public function testHandleEvent(string $clientId, array $headers, string $content, int $expectedEventsCount, int $expectedStatusCode, string $expectedContent, ?string $expectedException = NULL): void {
    $client = $this->getClientMock($clientId);
    $request = new Request(content: $content);
    $request->headers = new HeaderBag($headers);

    if ($expectedException !== NULL) {
      $this->expectException($expectedException);
    }

    $queueExpects = fn (MockObject $queue) => $queue
      ->expects($this->exactly($expectedEventsCount))
      ->method('createItem');

    $controller = $this->getEventHookController($queueExpects);
    $response = $controller->handleEvent($request);

    $this->assertSame($expectedStatusCode, $response->getStatusCode());
    $this->assertSame($expectedContent, $response->getContent());
  }

  /**
   * Data provider for ::testHandleVerification.
   */
  protected static function providerHandleVerification(): array {
    return [
      'OK' => [self::CLIENT_ID, ['x-okta-verification-challenge' => '123'], Response::HTTP_OK, '{"verification":"123"}'],
      'Missing header' => [self::CLIENT_ID, [], Response::HTTP_BAD_REQUEST, 'Missing Okta verification challenge.', BadRequestHttpException::class],
      'Invalid client' => ['foo_client', [], Response::HTTP_BAD_REQUEST, 'Invalid client.', BadRequestHttpException::class],
    ];
  }

  /**
   * Data provider for ::testHandleEvent.
   */
  protected static function providerHandleEvent(): array {
    $headers = [
      'Authorization' => 'Basic ' . self::EVENT_HOOK_SECRET,
      'Content-Type' => 'application/json',
    ];
    return [
      'OK' => [self::CLIENT_ID, $headers, static::sampleJson(), 2, Response::HTTP_NO_CONTENT, '[]'],
      'No events' => [self::CLIENT_ID, $headers, Json::encode(['data' => ['events' => []]]), 0, Response::HTTP_NO_CONTENT, '[]'],
      'Invalid JSON' => [self::CLIENT_ID, $headers, '', 0, Response::HTTP_BAD_REQUEST, 'Invalid JSON data.', BadRequestHttpException::class],
      'No authorization' => [self::CLIENT_ID, [], '', 0, Response::HTTP_BAD_REQUEST, 'Invalid authorization.', BadRequestHttpException::class],
      'Bad authorization' => [self::CLIENT_ID, ['Authorization' => 'Basic foo'], '', 0, Response::HTTP_BAD_REQUEST, 'Invalid authorization.', BadRequestHttpException::class],
    ];
  }

  /**
   * Gets the mocked OpenID Connect entity.
   */
  protected function getClientMock(string $id): OpenIDConnectClientEntityInterface {
    $client = $this->createMock(OpenIDConnectClientEntityInterface::class);
    $client->expects($this->any())
      ->method('id')
      ->willReturn($id);
    return $client;
  }

  /**
   * Gets the event hook controller.
   */
  protected function getEventHookController(?callable $queueExpects = NULL): EventHookController {
    $config = $this->createMock(Config::class);
    $config->expects($this->any())
      ->method('get')
      ->willReturnMap([
        ['client', self::CLIENT_ID],
        ['event_hook_secret', self::EVENT_HOOK_SECRET],
      ]);

    $configFactory = $this->createMock(ConfigFactoryInterface::class);
    $configFactory->expects($this->any())
      ->method('get')
      ->with(OktaplusSettingsForm::CONFIG_NAME)
      ->willReturn($config);

    $queue = $this->createMock(DatabaseQueue::class);
    $queueFactory = $this->createMock(QueueDatabaseFactory::class);
    $queueFactory->expects($this->any())
      ->method('get')
      ->with(OktaplusEventQueue::PLUGIN_ID)
      ->willReturn($queue);

    if ($queueExpects !== NULL) {
      $queueExpects($queue);
    }

    return new EventHookController($configFactory, $queueFactory);
  }

  /**
   * Sample JSON data.
   */
  protected static function sampleJson(): string {
    return Json::encode([
      'data' => [
        'events' => [
          [
            'uuid' => '38709aa2-2d64-49ce-ac0a-8d2ebc8da988',
            'eventType' => 'user.lifecycle.deactivate',
            'published' => '2024-07-30T00:00:00.000Z',
          ],
          [
            'uuid' => 'b327878f-c7cf-4ed3-826b-6ba03de4a864',
            'eventType' => 'user.lifecycle.deactivate',
            'published' => '2024-07-30T00:00:00.000Z',
          ],
        ],
      ],
    ]);
  }

}
