<?php

declare(strict_types=1);

namespace Drupal\Tests\oktaplus\Unit;

use Drupal\Core\Config\Config;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\oktaplus\Enum\EventOutcome;
use Drupal\oktaplus\Enum\EventType;
use Drupal\oktaplus\Enum\UserAddAction;
use Drupal\oktaplus\OktaLogEvent;
use Drupal\oktaplus\OktaplusUserUtility;
use Drupal\oktaplus\Plugin\QueueWorker\OktaplusEventQueue;
use Drupal\Tests\UnitTestCase;
use Drupal\user\UserInterface;
use Psr\Log\LoggerInterface;

/**
 * @coversDefaultClass \Drupal\oktaplus\Plugin\QueueWorker\OktaplusEventQueue
 */
final class EventQueueTest extends UnitTestCase {

  /**
   * @covers ::processItem
   * @dataProvider providerProcessItemAddEvents()
   */
  public function testProcessItemAddEvents(EventType $eventType, UserAddAction $action, bool $isCreated, bool $isActivated): void {
    $email = 'email@example.com';
    $uuid = $this->randomMachineName();
    $data = [
      'uuid' => $uuid,
      'eventType' => $eventType->value,
      'published' => '2024-07-30T00:00:00.000Z',
      'outcome' => [
        'result' => EventOutcome::Success->value,
      ],
      'target' => [
        [
          'id' => $this->randomMachineName(),
          'type' => 'User',
          'alternateId' => $email,
          'displayName' => $this->randomMachineName(),
        ],
        [
          'id' => '0oaz9fj21WKqTeaqs1d6',
          'type' => 'AppInstance',
        ],
      ],
    ];

    $user = $this->createMock(UserInterface::class);
    $logger = $this->createMock(LoggerInterface::class);
    $utility = $this->createMock(OktaplusUserUtility::class);
    $utility->expects($this->any())
      ->method('getTargetUser')->willReturn(NULL);
    $utility->expects($this->exactly($isCreated ? 1 : 0))
      ->method('createUserFromEvent')
      ->with(OktaLogEvent::fromArray($data))
      ->willReturn($user);
    $utility->expects($this->exactly($isActivated ? 1 : 0))
      ->method('activateUser')
      ->with($user, OktaLogEvent::fromArray($data));

    $config = $this->createMock(Config::class);
    $config->expects($this->any())
      ->method('get')->willReturnMap([
        ['app_instance_id', '0oaz9fj21WKqTeaqs1d6'],
        ['events', ['application_user_membership_add' => $action->value]],
      ]);
    $configFactory = $this->createMock(ConfigFactoryInterface::class);
    $configFactory->expects($this->any())
      ->method('get')->with('oktaplus.settings')->willReturn($config);

    $queue = new OktaplusEventQueue([], OktaplusEventQueue::PLUGIN_ID, [], $utility, $configFactory);
    $queue->setLogger($logger);
    $queue->processItem($data);
  }

  /**
   * @covers ::processItem
   * @dataProvider providerProcessItemBlockEvents
   */
  public function testProcessItemBlockEvents(EventType $eventType, bool $isBlocked, ?string $appInstance = NULL): void {
    $userId = 1000;
    $email = 'email@example.com';
    $uuid = $this->randomMachineName();

    $data = [
      'uuid' => $uuid,
      'eventType' => $eventType->value,
      'published' => '2024-07-30T00:00:00.000Z',
      'target' => [
        [
          'id' => $this->randomMachineName(),
          'type' => 'User',
        ],
      ],
      'outcome' => ['result' => 'SUCCESS'],
    ];

    if ($appInstance !== NULL) {
      $data['target'][] = [
        'id' => $appInstance,
        'type' => 'AppInstance',
      ];
    }

    $user = $this->createMock(UserInterface::class);
    $user->expects($this->any())->method('id')->willReturn($userId);
    $user->expects($this->any())->method('getEmail')->willReturn($email);

    $logger = $this->createMock(LoggerInterface::class);
    $utility = $this->createMock(OktaplusUserUtility::class);
    $utility->expects($this->any())
      ->method('getTargetUser')->willReturn($user);

    $expectCount = $isBlocked ? 1 : 0;
    $utility->expects($this->exactly($expectCount))
      ->method('blockUser')->with($user);
    $logger->expects($this->exactly($expectCount))
      ->method('notice')
      ->with('User @uid (@mail) was blocked via Okta event hook. Event type @eventType. Event uuid @uuid.', [
        '@uid' => $userId,
        '@mail' => $email,
        '@eventType' => $eventType->value,
        '@uuid' => $uuid,
      ]);

    $expectCount = match ($eventType) {
      EventType::UserLifecycleDeleteInitiated => 1,
      default => 0,
    };
    $utility->expects($this->exactly($expectCount))
      ->method('deleteUserFromAuthmap')->with($user);

    $config = $this->createMock(Config::class);
    $config->expects($this->any())
      ->method('get')->with('app_instance_id')->willReturn('0oaz9fj21WKqTeaqs1d6');
    $configFactory = $this->createMock(ConfigFactoryInterface::class);
    $configFactory->expects($this->any())
      ->method('get')->with('oktaplus.settings')->willReturn($config);
    $queue = new OktaplusEventQueue([], OktaplusEventQueue::PLUGIN_ID, [], $utility, $configFactory);
    $queue->setLogger($logger);
    $queue->processItem($data);
  }

  /**
   * Test unsuccessful event.
   */
  public function testUnsuccessfulEvent(): void {
    $uuid = $this->randomMachineName();
    $logger = $this->createMock(LoggerInterface::class);
    $logger->expects($this->once())
      ->method('warning')
      ->with('Event outcome was not successful. Event type @eventType. Event UUID: @uuid. Reason: @reason.', [
        '@eventType' => EventType::UserLifecycleDeactivate->value,
        '@uuid' => $uuid,
        '@reason' => 'Catastrophic failure',
      ]);
    $queue = new OktaplusEventQueue(
      configuration: [],
      plugin_id: OktaplusEventQueue::PLUGIN_ID,
      plugin_definition: [],
      oktaplus: $this->createMock(OktaplusUserUtility::class),
      configFactory: $this->createMock(ConfigFactoryInterface::class),
    );
    $queue->setLogger($logger);
    $queue->processItem([
      'uuid' => $uuid,
      'eventType' => EventType::UserLifecycleDeactivate->value,
      'published' => '2024-07-30T00:00:00.000Z',
      'outcome' => [
        'result' => EventOutcome::Failure->value,
        'reason' => 'Catastrophic failure',
      ],
    ]);
  }

  /**
   * Data provider for ::testProcessItemAddEvents.
   */
  protected static function providerProcessItemAddEvents(): array {
    return [
      [EventType::ApplicationUserMembershipAdd, UserAddAction::Ignore, FALSE, FALSE],
      [EventType::ApplicationUserMembershipAdd, UserAddAction::Create, TRUE, FALSE],
      [EventType::ApplicationUserMembershipAdd, UserAddAction::Activate, TRUE, TRUE],
    ];
  }

  /**
   * Data provider for ::testProcessItemBlockEvents.
   */
  protected static function providerProcessItemBlockEvents(): array {
    return [
      [EventType::ApplicationUserMembershipRemove, FALSE],
      [EventType::ApplicationUserMembershipRemove, FALSE, 'zzz'],
      [EventType::ApplicationUserMembershipRemove, TRUE, '0oaz9fj21WKqTeaqs1d6'],
      [EventType::UserAccountUpdateProfile, FALSE],
      [EventType::UserLifecycleSuspend, TRUE],
      [EventType::UserLifecycleUnsuspend, FALSE],
      [EventType::UserLifecycleActivate, FALSE],
      [EventType::UserLifecycleReactivate, FALSE],
      [EventType::UserLifecycleDeactivate, TRUE],
      [EventType::UserLifecycleDeleteInitiated, TRUE],
      [EventType::UserSessionEnd, FALSE],
    ];
  }

}
