<?php

declare(strict_types=1);

namespace Drupal\Tests\oktaplus\Unit;

use Drupal\Core\Config\Config;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\oktaplus\Enum\EventOutcome;
use Drupal\oktaplus\Enum\EventType;
use Drupal\oktaplus\OktaplusUserUtility;
use Drupal\oktaplus\Plugin\QueueWorker\OktaplusEventQueue;
use Drupal\Tests\UnitTestCase;
use Drupal\user\UserInterface;
use Psr\Log\LoggerInterface;

/**
 * EventQueueTest is getting busy. Let's split it up.
 */
final class GroupUserMembershipTest extends UnitTestCase {

  /**
   * Test user is added to group(s).
   *
   * @dataProvider providerGroupMembership
   */
  public function testGroupMembership(EventType $eventType, string $method, string $action): void {
    $email = 'email@example.com';
    $data = [
      'uuid' => '4cc38455-c34f-4546-a436-9902c3f7b8ff',
      'eventType' => $eventType->value,
      'published' => '2021-01-01T00:00:00.000Z',
      'outcome' => [
        'result' => EventOutcome::Success->value,
      ],
      'target' => [
        [
          'type' => 'UserGroup',
          'displayName' => 'OKTA_SUPER_CUSTOMER',
        ],
        [
          'type' => 'User',
          'alternateId' => $email,
        ],
      ],
    ];
    $user = $this->createMock(UserInterface::class);
    $user->expects($this->any())->method('id')->willReturn(1000);
    $user->expects($this->any())->method('getEmail')->willReturn($email);
    $utility = $this->createMock(OktaplusUserUtility::class);
    $utility->expects($this->any())
      ->method('getTargetUser')->willReturn($user);
    $utility->expects($this->once())
      ->method($method)
      ->with($user, ['customer', 'super_customer']);
    $config = $this->createMock(Config::class);
    $config->expects($this->once())
      ->method('get')
      ->with('role_mappings')
      ->willReturn([
        'customer' => ['OKTA_CUSTOMER', 'OKTA_SUPER_CUSTOMER'],
        'super_customer' => ['OKTA_SUPER_CUSTOMER'],
        'admin' => ['OKTA_ADMIN'],
      ]);
    $configFactory = $this->createMock(ConfigFactoryInterface::class);
    $configFactory->expects($this->once())
      ->method('get')
      ->with('openid_connect.settings')
      ->willReturn($config);
    $logger = $this->createMock(LoggerInterface::class);
    $logger->expects($this->once())
      ->method('notice')
      ->with('User @uid (@mail) had role(s) (@roles) ' . $action . ' via Okta event hook. Event type @eventType. Event uuid @uuid.', [
        '@uid' => 1000,
        '@mail' => 'email@example.com',
        '@roles' => 'customer, super_customer',
        '@eventType' => $eventType->value,
        '@uuid' => '4cc38455-c34f-4546-a436-9902c3f7b8ff',
      ]);
    $queue = new OktaplusEventQueue([], OktaplusEventQueue::PLUGIN_ID, [], $utility, $configFactory);
    $queue->setLogger($logger);
    $queue->processItem($data);
  }

  /**
   * @see testGroupMembership
   */
  public static function providerGroupMembership(): array {
    return [
      [EventType::GroupUserMembershipAdd, 'addUserRoles', 'added'],
      [EventType::GroupUserMembershipRemove, 'removeUserRoles', 'removed'],
    ];
  }

  /**
   * Test user is not fetched when group not found in mapping.
   *
   * @dataProvider providerEventType
   */
  public function testGroupWithNoMapping(EventType $eventType): void {
    $data = [
      'uuid' => '4cc38455-c34f-4546-a436-9902c3f7b8ff',
      'eventType' => $eventType->value,
      'published' => '2021-01-01T00:00:00.000Z',
      'outcome' => [
        'result' => EventOutcome::Success->value,
      ],
      'target' => [
        [
          'type' => 'UserGroup',
          'displayName' => 'OKTA_MEMBERS',
        ],
      ],
    ];

    $utility = $this->createMock(OktaplusUserUtility::class);
    $utility->expects($this->never())
      ->method('getTargetUser');

    $config = $this->createMock(Config::class);
    $config->expects($this->once())
      ->method('get')
      ->with('role_mappings')
      ->willReturn([
        'customer' => ['OKTA_CUSTOMER'],
      ]);
    $configFactory = $this->createMock(ConfigFactoryInterface::class);
    $configFactory->expects($this->once())
      ->method('get')
      ->with('openid_connect.settings')
      ->willReturn($config);
    $queue = new OktaplusEventQueue([], OktaplusEventQueue::PLUGIN_ID, [], $utility, $configFactory);
    $queue->processItem($data);
  }

  /**
   * @see testGroupWithNoMapping
   */
  public static function providerEventType(): array {
    return [
      [EventType::GroupUserMembershipAdd],
      [EventType::GroupUserMembershipRemove],
    ];
  }

}
