<?php

declare(strict_types=1);

namespace Drupal\Tests\oktaplus\Unit;

use Drupal\oktaplus\Enum\EventOutcome;
use Drupal\oktaplus\Enum\EventType;
use Drupal\oktaplus\Exception\InvalidLogEventException;
use Drupal\oktaplus\Exception\UnhandledLogEventTypeException;
use Drupal\oktaplus\OktaLogEvent;
use Drupal\Tests\UnitTestCase;

/**
 * @coversDefaultClass \Drupal\oktaplus\OktaLogEvent
 */
final class OktaLogEventTest extends UnitTestCase {

  /**
   * @covers ::fromArray
   */
  public function testFromArray(): void {
    $uuid = '4cc38455-c34f-4546-a436-9902c3f7b8ff';
    $event = OktaLogEvent::fromArray([
      'uuid' => $uuid,
      'eventType' => EventType::UserLifecycleDeactivate->value,
      'published' => '2021-01-01T00:00:00.000Z',
      'outcome' => [
        'result' => EventOutcome::Success->value,
      ],
    ]);
    $this->assertSame($uuid, $event->uuid);
    $this->assertSame(EventType::UserLifecycleDeactivate, $event->eventType);
    $this->assertSame(1609419600, $event->published->getTimestamp());
    $this->assertSame(EventOutcome::Success, $event->eventOutcome);
  }

  /**
   * @covers ::fromArray
   */
  public function testUnhandledLogEvent(): void {
    $uuid = '4cc38455-c34f-4546-a436-9902c3f7b8ff';
    $this->expectException(UnhandledLogEventTypeException::class);
    $this->expectExceptionMessage("Unhandled event type unknown. Event UUID: $uuid");
    $event = OktaLogEvent::fromArray([
      'uuid' => $uuid,
      'eventType' => 'unknown',
      'published' => '2021-01-01T00:00:00.000Z',
      'outcome' => [
        'result' => EventOutcome::Success->value,
      ],
    ]);
    $this->assertInstanceOf(OktaLogEvent::class, $event);
  }

  /**
   * @covers ::fromArray
   * @dataProvider providerInvalidLogEvent
   */
  public function testInvalidLogEvent(array $data, string $expectedException): void {
    $this->expectException(InvalidLogEventException::class);
    $this->expectExceptionMessage($expectedException);
    $event = OktaLogEvent::fromArray($data);
    $this->assertInstanceOf(OktaLogEvent::class, $event);
  }

  /**
   * @covers ::getTargetAppInstance
   */
  public function testGetTargetAppInstance(): void {
    $event = OktaLogEvent::fromArray([
      'uuid' => $this->randomMachineName(),
      'eventType' => EventType::UserLifecycleDeactivate->value,
      'published' => '2021-01-01T00:00:00.000Z',
      'outcome' => ['result' => EventOutcome::Success->value],
      'target' => [
        ['id' => '00u1x5zj4zZ2z1b4', 'type' => 'User'],
        ['type' => 'AppInstance', 'id' => '0oaz9fj21WKqTeaqs1d6'],
      ],
    ]);
    $targetAppInstance = $event->getTargetAppInstance();
    $this->assertIsArray($targetAppInstance);
    $this->assertArrayHasKey('id', $targetAppInstance);
    $this->assertSame('0oaz9fj21WKqTeaqs1d6', $targetAppInstance['id']);
  }

  /**
   * @covers ::getTargetUser
   */
  public function testGetTargetUser(): void {
    $event = OktaLogEvent::fromArray([
      'uuid' => $this->randomMachineName(),
      'eventType' => EventType::UserLifecycleDeactivate->value,
      'published' => '2021-01-01T00:00:00.000Z',
      'outcome' => ['result' => EventOutcome::Success->value],
      'target' => [
        ['type' => 'NotUser'],
        ['id' => '00u1x5zj4zZ2z1b4', 'type' => 'User'],
        ['type' => 'StillNotUser'],
      ],
    ]);
    $targetUser = $event->getTargetUser();
    $this->assertIsArray($targetUser);
    $this->assertArrayHasKey('id', $targetUser);
    $this->assertSame('00u1x5zj4zZ2z1b4', $targetUser['id']);
  }

  /**
   * @covers ::getTargetGroup
   */
  public function testGetTargetGroup(): void {
    $event = OktaLogEvent::fromArray([
      'uuid' => $this->randomMachineName(),
      'eventType' => EventType::GroupUserMembershipAdd->value,
      'published' => '2021-01-01T00:00:00.000Z',
      'outcome' => ['result' => EventOutcome::Success->value],
      'target' => [
        ['type' => 'NotGroup'],
        ['id' => '00u1x5zj4zZ2z1b4', 'type' => 'UserGroup'],
        ['type' => 'StillNotGroup'],
      ],
    ]);
    $group = $event->getTargetGroup();
    $this->assertIsArray($group);
    $this->assertArrayHasKey('id', $group);
    $this->assertSame('00u1x5zj4zZ2z1b4', $group['id']);
  }

  /**
   * Data provider for ::testInvalidLogEvent.
   */
  protected static function providerInvalidLogEvent(): array {
    $uuid = '4cc38455-c34f-4546-a436-9902c3f7b8ff';
    $data = [
      'uuid' => $uuid,
      'eventType' => EventType::UserLifecycleDeactivate->value,
      'published' => '2021-01-01T00:00:00.000Z',
      'outcome' => [
        'result' => EventOutcome::Success->value,
      ],
    ];
    return [
      'Invalid UUID' => [array_diff_key($data, ['uuid' => FALSE]), 'Missing event UUID.'],
      'Missing eventType' => [array_diff_key($data, ['eventType' => FALSE]), "Missing eventType. Event UUID: $uuid"],
      'Missing published date' => [array_diff_key($data, ['published' => FALSE]), "Missing published date. Event UUID: $uuid"],
      'Invalid date format' => [['published' => '2021-01-01'] + $data, "Invalid date format 2021-01-01. Event UUID: $uuid"],
      'Missing outcome' => [array_diff_key($data, ['outcome' => FALSE]), "Missing outcome. Event UUID: $uuid"],
    ];
  }

}
